﻿-----------------------------------------------------------------------------
--
-- data
--   \crowdControl
--     \<playerName>
--       - count
--       \detailList (array)
--         - time, targetName, abilityName
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentCrowdControl")
L:RegisterTranslations("enUS", function() return {
	["Crowd Controllers List"] = true,
	["Crowd Control"] = true,
	["Crowd Control Details"] = true,

	["Crowd Control Options."] = true,
	["Detailed Data"] = true,
	["Gather detailed information on crowd control."] = true,
} end)

local BS = AceLibrary("Babble-Spell-2.2")

-----------------------------------------------------------------------------

local AssessmentCrowdControl = Assessment:NewModule("CrowdControl", "AceEvent-2.0", "Parser-3.0")
AssessmentCrowdControl.db = Assessment:AcquireDBNamespace("CrowdControl")
Assessment:RegisterDefaults("CrowdControl", "profile", {
	enable					= true,
	gatherDetailDataFilter	= "GROUP",
})

-----------------------------------------------------------------------------

local profile
local gatherDetailDataMethod

local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.CrowdControlCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local crowdControl = fightData.crowdControl
				if not crowdControl then return 0 end
				
				local playerCrowdControl = crowdControl[data.key]
				if not playerCrowdControl then return 0 end

				return playerCrowdControl.count
			 end,
	data = "crowdControl",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.CrowdControllers =
{
	method = function(fightData)
				if not fightData.crowdControl then return emptyMethod end
				return pairs(fightData.crowdControl)
			 end,
	data = "crowdControl",
	description = L["Crowd Controllers List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.CrowdControlDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Crowd Control"]),
	listType			= "EnumerateTable",
	listDataTag			= "crowdControl,[WindowParameter(1)],detailList",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[GetValueField(\"targetName\")] [GetValueField(\"abilityName\"):Bracket]" }, { tag = "[GetValueField(\"time\"):FormatTime]" } },
	description			= L["Crowd Control Details"],
	selectable			= false,
	defaultColor		= { 0.3, 0.3, 0.8 },
	favorite			= false,
}

Assessment.DefaultDisplaySets.CrowdControl =
{
	titleTag			= L["Crowd Control"],
	listType			= "GroupMembers", 
	barValueTag			= "CrowdControlCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 0,
	description			= L["Crowd Control"],
	defaultColor		= { 0.3, 0.3, 0.8 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Crowd Control Details"],
	onClickData			= "[Key]",
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.crowdControl = "crowdControl"

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.crowdControl =
{
	type = "group",
	name = L["Crowd Control"],
	desc = L["Crowd Control Options."],
	args =
	{
		gatherDetailsData =
		{
			type = "text",
			name = L["Detailed Data"],
			desc = L["Gather detailed information on crowd control."],
			get = function() return profile.gatherDetailDataFilter end,
			set = function(v) profile.gatherDetailDataFilter = v AssessmentCrowdControl:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentCrowdControl:OnInitialize()
	profile = self.db.profile
	gatherDetailDataMethod = Assessment.GatherFilterList[profile.gatherDetailDataFilter].method
end

function AssessmentCrowdControl:OnEnable()
	self:RegisterParserEvent({ eventType = "Cast" }, "OnParserCastEvent")
end

function AssessmentCrowdControl:OnDisable()
end

-----------------------------------------------------------------------------

local crowdControlSpells =
{
	[BS["Shackle Undead"]]			= true,
	[BS["Hibernate"]]				= true,
	[BS["Sap"]]						= true,
	[BS["Polymorph"]]				= true,
	[BS["Polymorph: Pig"]]			= true,
	[BS["Polymorph: Turtle"]]		= true,
	[BS["Freezing Trap Effect"]]	= true,
}

-----------------------------------------------------------------------------

function AssessmentCrowdControl:OnParserCastEvent(eventData)
	if eventData.isBegin then return end

	local sourceName = eventData.sourceName
	if not sourceName then return end

	local recipientName = eventData.recipientName
	if not recipientName then return end
	
	local abilityName = eventData.abilityName
	if not crowdControlSpells[abilityName] then return end

	local crowdControlTime = time()
	local time = GetTime()

	for data in self:IterateOOCUpdateDataTables() do
		data.dataUpdateTimes.crowdControl = time
		local crowdControl = data.crowdControl
		if not crowdControl then
			crowdControl = new()
			data.crowdControl = crowdControl
		end

		local playerCrowdControl = crowdControl[sourceName]
		if not playerCrowdControl then
			playerCrowdControl = new()
			playerCrowdControl.count = 1
			crowdControl[sourceName] = playerCrowdControl
		else
			playerCrowdControl.count = playerCrowdControl.count + 1
		end
		
		if gatherDetailDataMethod(sourceName) then
			local detailList = playerCrowdControl.detailList
			if not detailList then
				detailList = new()
				playerCrowdControl.detailList = detailList
			end

			local details = new()
			details.time = crowdControlTime
			details.targetName = recipientName
			details.abilityName = abilityName
			
			detailList[#detailList+1] = details
		end
	end
end

-----------------------------------------------------------------------------
